@@ -1,21 +1,72 @@
 use crate::{
     render_resource::{
-        AsModuleDescriptorError, BindGroupLayout, BindGroupLayoutId, ProcessShaderError,
-        RawFragmentState, RawRenderPipelineDescriptor, RawVertexState, RenderPipeline,
-        RenderPipelineDescriptor, Shader, ShaderImport, ShaderProcessor, ShaderReflectError,
+        AsModuleDescriptorError, BindGroupLayout, BindGroupLayoutId, ComputePipeline,
+        ComputePipelineDescriptor, ProcessShaderError, ProcessedShader,
+        RawComputePipelineDescriptor, RawFragmentState, RawRenderPipelineDescriptor,
+        RawVertexState, RenderPipeline, RenderPipelineDescriptor, Shader, ShaderImport,
+        ShaderProcessor, ShaderReflectError,
     },
     renderer::RenderDevice,
     RenderWorld,
 };
-use bevy_app::EventReader;
 use bevy_asset::{AssetEvent, Assets, Handle};
+use bevy_ecs::event::EventReader;
 use bevy_ecs::system::{Res, ResMut};
-use bevy_utils::{tracing::error, HashMap, HashSet};
-use std::{collections::hash_map::Entry, hash::Hash, ops::Deref, sync::Arc};
+use bevy_utils::{default, tracing::error, Entry, HashMap, HashSet};
+use std::{hash::Hash, mem, ops::Deref, sync::Arc};
 use thiserror::Error;
-use wgpu::{PipelineLayoutDescriptor, ShaderModule, VertexBufferLayout};
+use wgpu::{PipelineLayoutDescriptor, ShaderModule, VertexBufferLayout as RawVertexBufferLayout};
 
-use super::ProcessedShader;
+enum PipelineDescriptor {
+    RenderPipelineDescriptor(Box<RenderPipelineDescriptor>),
+    ComputePipelineDescriptor(Box<ComputePipelineDescriptor>),
+}
+
+#[derive(Debug)]
+pub enum Pipeline {
+    RenderPipeline(RenderPipeline),
+    ComputePipeline(ComputePipeline),
+}
+
+type CachedPipelineId = usize;
+
+#[derive(Copy, Clone, Debug, Hash, Eq, PartialEq)]
+pub struct CachedRenderPipelineId(CachedPipelineId);
+
+impl CachedRenderPipelineId {
+    pub const INVALID: Self = CachedRenderPipelineId(usize::MAX);
+}
+
+#[derive(Copy, Clone, Debug, Hash, Eq, PartialEq)]
+pub struct CachedComputePipelineId(CachedPipelineId);
+
+impl CachedComputePipelineId {
+    pub const INVALID: Self = CachedComputePipelineId(usize::MAX);
+}
+
+struct CachedPipeline {
+    descriptor: PipelineDescriptor,
+    state: CachedPipelineState,
+}
+
+#[derive(Debug)]
+pub enum CachedPipelineState {
+    Queued,
+    Ok(Pipeline),
+    Err(PipelineCacheError),
+}
+
+impl CachedPipelineState {
+    pub fn unwrap(&self) -> &Pipeline {
+        match self {
+            CachedPipelineState::Ok(pipeline) => pipeline,
+            CachedPipelineState::Queued => {
+                panic!("Pipeline has not been compiled yet. It is still in the 'Queued' state.")
+            }
+            CachedPipelineState::Err(err) => panic!("{}", err),
+        }
+    }
+}
 
 #[derive(Default)]
 pub struct ShaderData {
@@ -25,13 +76,6 @@ pub struct ShaderData {
     dependents: HashSet<Handle<Shader>>,
 }
 
-#[derive(Copy, Clone, Debug, Hash, Eq, PartialEq)]
-pub struct CachedPipelineId(usize);
-
-impl CachedPipelineId {
-    pub const INVALID: Self = CachedPipelineId(usize::MAX);
-}
-
 #[derive(Default)]
 struct ShaderCache {
     data: HashMap<Handle<Shader>, ShaderData>,
@@ -48,11 +92,11 @@ impl ShaderCache {
         pipeline: CachedPipelineId,
         handle: &Handle<Shader>,
         shader_defs: &[String],
-    ) -> Result<Arc<ShaderModule>, RenderPipelineError> {
+    ) -> Result<Arc<ShaderModule>, PipelineCacheError> {
         let shader = self
             .shaders
             .get(handle)
-            .ok_or_else(|| RenderPipelineError::ShaderNotLoaded(handle.clone_weak()))?;
+            .ok_or_else(|| PipelineCacheError::ShaderNotLoaded(handle.clone_weak()))?;
         let data = self.data.entry(handle.clone_weak()).or_default();
         let n_asset_imports = shader
             .imports()
@@ -64,7 +108,7 @@ impl ShaderCache {
             .filter(|import| matches!(import, ShaderImport::AssetPath(_)))
             .count();
         if n_asset_imports != n_resolved_asset_imports {
-            return Err(RenderPipelineError::ShaderImportNotYetAvailable);
+            return Err(PipelineCacheError::ShaderImportNotYetAvailable);
         }
 
         data.pipelines.insert(pipeline);
@@ -82,12 +126,27 @@ impl ShaderCache {
                 let module_descriptor = match processed.get_module_descriptor() {
                     Ok(module_descriptor) => module_descriptor,
                     Err(err) => {
-                        return Err(RenderPipelineError::AsModuleDescriptorError(err, processed));
+                        return Err(PipelineCacheError::AsModuleDescriptorError(err, processed));
                     }
                 };
-                entry.insert(Arc::new(
-                    render_device.create_shader_module(&module_descriptor),
-                ))
+
+                render_device
+                    .wgpu_device()
+                    .push_error_scope(wgpu::ErrorFilter::Validation);
+                let shader_module = render_device.create_shader_module(&module_descriptor);
+                let error = render_device.wgpu_device().pop_error_scope();
+
+                // `now_or_never` will return Some if the future is ready and None otherwise.
+                // On native platforms, wgpu will yield the error immediatly while on wasm it may take longer since the browser APIs are asynchronous.
+                // So to keep the complexity of the ShaderCache low, we will only catch this error early on native platforms,
+                // and on wasm the error will be handled by wgpu and crash the application.
+                if let Some(Some(wgpu::Error::Validation { description, .. })) =
+                    bevy_utils::futures::now_or_never(error)
+                {
+                    return Err(PipelineCacheError::CreateShaderModule(description));
+                }
+
+                entry.insert(Arc::new(shader_module))
             }
         };
 
@@ -176,13 +235,13 @@ impl LayoutCache {
                 .collect::<Vec<_>>();
             render_device.create_pipeline_layout(&PipelineLayoutDescriptor {
                 bind_group_layouts: &bind_group_layouts,
-                ..Default::default()
+                ..default()
             })
         })
     }
 }
 
-pub struct RenderPipelineCache {
+pub struct PipelineCache {
     layout_cache: LayoutCache,
     shader_cache: ShaderCache,
     device: RenderDevice,
@@ -190,83 +249,101 @@ pub struct RenderPipelineCache {
     waiting_pipelines: HashSet<CachedPipelineId>,
 }
 
-struct CachedPipeline {
-    descriptor: RenderPipelineDescriptor,
-    state: CachedPipelineState,
-}
-
-#[derive(Debug)]
-pub enum CachedPipelineState {
-    Queued,
-    Ok(RenderPipeline),
-    Err(RenderPipelineError),
-}
-
-impl CachedPipelineState {
-    pub fn unwrap(&self) -> &RenderPipeline {
-        match self {
-            CachedPipelineState::Ok(pipeline) => pipeline,
-            CachedPipelineState::Queued => {
-                panic!("Pipeline has not been compiled yet. It is still in the 'Queued' state.")
-            }
-            CachedPipelineState::Err(err) => panic!("{}", err),
-        }
-    }
-}
-
-#[derive(Error, Debug)]
-pub enum RenderPipelineError {
-    #[error(
-        "Pipeline cound not be compiled because the following shader is not loaded yet: {0:?}"
-    )]
-    ShaderNotLoaded(Handle<Shader>),
-    #[error(transparent)]
-    ProcessShaderError(#[from] ProcessShaderError),
-    #[error("{0}")]
-    AsModuleDescriptorError(AsModuleDescriptorError, ProcessedShader),
-    #[error("Shader import not yet available.")]
-    ShaderImportNotYetAvailable,
-}
-
-impl RenderPipelineCache {
+impl PipelineCache {
     pub fn new(device: RenderDevice) -> Self {
         Self {
             device,
-            layout_cache: Default::default(),
-            shader_cache: Default::default(),
-            waiting_pipelines: Default::default(),
-            pipelines: Default::default(),
+            layout_cache: default(),
+            shader_cache: default(),
+            waiting_pipelines: default(),
+            pipelines: default(),
         }
     }
 
     #[inline]
-    pub fn get_state(&self, id: CachedPipelineId) -> &CachedPipelineState {
+    pub fn get_render_pipeline_state(&self, id: CachedRenderPipelineId) -> &CachedPipelineState {
         &self.pipelines[id.0].state
     }
 
     #[inline]
-    pub fn get(&self, id: CachedPipelineId) -> Option<&RenderPipeline> {
-        if let CachedPipelineState::Ok(pipeline) = &self.pipelines[id.0].state {
+    pub fn get_compute_pipeline_state(&self, id: CachedComputePipelineId) -> &CachedPipelineState {
+        &self.pipelines[id.0].state
+    }
+
+    #[inline]
+    pub fn get_render_pipeline_descriptor(
+        &self,
+        id: CachedRenderPipelineId,
+    ) -> &RenderPipelineDescriptor {
+        match &self.pipelines[id.0].descriptor {
+            PipelineDescriptor::RenderPipelineDescriptor(descriptor) => descriptor,
+            PipelineDescriptor::ComputePipelineDescriptor(_) => unreachable!(),
+        }
+    }
+
+    #[inline]
+    pub fn get_compute_pipeline_descriptor(
+        &self,
+        id: CachedComputePipelineId,
+    ) -> &ComputePipelineDescriptor {
+        match &self.pipelines[id.0].descriptor {
+            PipelineDescriptor::RenderPipelineDescriptor(_) => unreachable!(),
+            PipelineDescriptor::ComputePipelineDescriptor(descriptor) => descriptor,
+        }
+    }
+
+    #[inline]
+    pub fn get_render_pipeline(&self, id: CachedRenderPipelineId) -> Option<&RenderPipeline> {
+        if let CachedPipelineState::Ok(Pipeline::RenderPipeline(pipeline)) =
+            &self.pipelines[id.0].state
+        {
             Some(pipeline)
         } else {
             None
         }
     }
 
-    pub fn queue(&mut self, descriptor: RenderPipelineDescriptor) -> CachedPipelineId {
-        let id = CachedPipelineId(self.pipelines.len());
+    #[inline]
+    pub fn get_compute_pipeline(&self, id: CachedComputePipelineId) -> Option<&ComputePipeline> {
+        if let CachedPipelineState::Ok(Pipeline::ComputePipeline(pipeline)) =
+            &self.pipelines[id.0].state
+        {
+            Some(pipeline)
+        } else {
+            None
+        }
+    }
+
+    pub fn queue_render_pipeline(
+        &mut self,
+        descriptor: RenderPipelineDescriptor,
+    ) -> CachedRenderPipelineId {
+        let id = CachedRenderPipelineId(self.pipelines.len());
         self.pipelines.push(CachedPipeline {
-            descriptor,
+            descriptor: PipelineDescriptor::RenderPipelineDescriptor(Box::new(descriptor)),
             state: CachedPipelineState::Queued,
         });
-        self.waiting_pipelines.insert(id);
+        self.waiting_pipelines.insert(id.0);
+        id
+    }
+
+    pub fn queue_compute_pipeline(
+        &mut self,
+        descriptor: ComputePipelineDescriptor,
+    ) -> CachedComputePipelineId {
+        let id = CachedComputePipelineId(self.pipelines.len());
+        self.pipelines.push(CachedPipeline {
+            descriptor: PipelineDescriptor::ComputePipelineDescriptor(Box::new(descriptor)),
+            state: CachedPipelineState::Queued,
+        });
+        self.waiting_pipelines.insert(id.0);
         id
     }
 
     fn set_shader(&mut self, handle: &Handle<Shader>, shader: &Shader) {
         let pipelines_to_queue = self.shader_cache.set_shader(handle, shader.clone());
         for cached_pipeline in pipelines_to_queue {
-            self.pipelines[cached_pipeline.0].state = CachedPipelineState::Queued;
+            self.pipelines[cached_pipeline].state = CachedPipelineState::Queued;
             self.waiting_pipelines.insert(cached_pipeline);
         }
     }
@@ -274,114 +351,172 @@ impl RenderPipelineCache {
     fn remove_shader(&mut self, shader: &Handle<Shader>) {
         let pipelines_to_queue = self.shader_cache.remove(shader);
         for cached_pipeline in pipelines_to_queue {
-            self.pipelines[cached_pipeline.0].state = CachedPipelineState::Queued;
+            self.pipelines[cached_pipeline].state = CachedPipelineState::Queued;
             self.waiting_pipelines.insert(cached_pipeline);
         }
     }
 
+    fn process_render_pipeline(
+        &mut self,
+        id: CachedPipelineId,
+        descriptor: &RenderPipelineDescriptor,
+    ) -> CachedPipelineState {
+        let vertex_module = match self.shader_cache.get(
+            &self.device,
+            id,
+            &descriptor.vertex.shader,
+            &descriptor.vertex.shader_defs,
+        ) {
+            Ok(module) => module,
+            Err(err) => {
+                return CachedPipelineState::Err(err);
+            }
+        };
+
+        let fragment_data = if let Some(fragment) = &descriptor.fragment {
+            let fragment_module = match self.shader_cache.get(
+                &self.device,
+                id,
+                &fragment.shader,
+                &fragment.shader_defs,
+            ) {
+                Ok(module) => module,
+                Err(err) => {
+                    return CachedPipelineState::Err(err);
+                }
+            };
+            Some((
+                fragment_module,
+                fragment.entry_point.deref(),
+                &fragment.targets,
+            ))
+        } else {
+            None
+        };
+
+        let vertex_buffer_layouts = descriptor
+            .vertex
+            .buffers
+            .iter()
+            .map(|layout| RawVertexBufferLayout {
+                array_stride: layout.array_stride,
+                attributes: &layout.attributes,
+                step_mode: layout.step_mode,
+            })
+            .collect::<Vec<_>>();
+
+        let layout = if let Some(layout) = &descriptor.layout {
+            Some(self.layout_cache.get(&self.device, layout))
+        } else {
+            None
+        };
+
+        let descriptor = RawRenderPipelineDescriptor {
+            multiview: None,
+            depth_stencil: descriptor.depth_stencil.clone(),
+            label: descriptor.label.as_deref(),
+            layout,
+            multisample: descriptor.multisample,
+            primitive: descriptor.primitive,
+            vertex: RawVertexState {
+                buffers: &vertex_buffer_layouts,
+                entry_point: descriptor.vertex.entry_point.deref(),
+                module: &vertex_module,
+            },
+            fragment: fragment_data
+                .as_ref()
+                .map(|(module, entry_point, targets)| RawFragmentState {
+                    entry_point,
+                    module,
+                    targets,
+                }),
+        };
+
+        let pipeline = self.device.create_render_pipeline(&descriptor);
+
+        CachedPipelineState::Ok(Pipeline::RenderPipeline(pipeline))
+    }
+
+    fn process_compute_pipeline(
+        &mut self,
+        id: CachedPipelineId,
+        descriptor: &ComputePipelineDescriptor,
+    ) -> CachedPipelineState {
+        let compute_module = match self.shader_cache.get(
+            &self.device,
+            id,
+            &descriptor.shader,
+            &descriptor.shader_defs,
+        ) {
+            Ok(module) => module,
+            Err(err) => {
+                return CachedPipelineState::Err(err);
+            }
+        };
+
+        let layout = if let Some(layout) = &descriptor.layout {
+            Some(self.layout_cache.get(&self.device, layout))
+        } else {
+            None
+        };
+
+        let descriptor = RawComputePipelineDescriptor {
+            label: descriptor.label.as_deref(),
+            layout,
+            module: &compute_module,
+            entry_point: descriptor.entry_point.as_ref(),
+        };
+
+        let pipeline = self.device.create_compute_pipeline(&descriptor);
+
+        CachedPipelineState::Ok(Pipeline::ComputePipeline(pipeline))
+    }
+
     pub fn process_queue(&mut self) {
-        let pipelines = std::mem::take(&mut self.waiting_pipelines);
-        for id in pipelines {
-            let state = &mut self.pipelines[id.0];
-            match &state.state {
+        let waiting_pipelines = mem::take(&mut self.waiting_pipelines);
+        let mut pipelines = mem::take(&mut self.pipelines);
+
+        for id in waiting_pipelines {
+            let pipeline = &mut pipelines[id];
+            match &pipeline.state {
                 CachedPipelineState::Ok(_) => continue,
                 CachedPipelineState::Queued => {}
                 CachedPipelineState::Err(err) => {
                     match err {
-                        RenderPipelineError::ShaderNotLoaded(_)
-                        | RenderPipelineError::ShaderImportNotYetAvailable => { /* retry */ }
+                        PipelineCacheError::ShaderNotLoaded(_)
+                        | PipelineCacheError::ShaderImportNotYetAvailable => { /* retry */ }
                         // shader could not be processed ... retrying won't help
-                        RenderPipelineError::ProcessShaderError(err) => {
+                        PipelineCacheError::ProcessShaderError(err) => {
                             error!("failed to process shader: {}", err);
                             continue;
                         }
-                        RenderPipelineError::AsModuleDescriptorError(err, source) => {
+                        PipelineCacheError::AsModuleDescriptorError(err, source) => {
                             log_shader_error(source, err);
                             continue;
                         }
+                        PipelineCacheError::CreateShaderModule(description) => {
+                            error!("failed to create shader module: {}", description);
+                            continue;
+                        }
                     }
                 }
             }
 
-            let descriptor = &state.descriptor;
-            let vertex_module = match self.shader_cache.get(
-                &self.device,
-                id,
-                &descriptor.vertex.shader,
-                &descriptor.vertex.shader_defs,
-            ) {
-                Ok(module) => module,
-                Err(err) => {
-                    state.state = CachedPipelineState::Err(err);
-                    self.waiting_pipelines.insert(id);
-                    continue;
+            pipeline.state = match &pipeline.descriptor {
+                PipelineDescriptor::RenderPipelineDescriptor(descriptor) => {
+                    self.process_render_pipeline(id, descriptor)
+                }
+                PipelineDescriptor::ComputePipelineDescriptor(descriptor) => {
+                    self.process_compute_pipeline(id, descriptor)
                 }
             };
 
-            let fragment_data = if let Some(fragment) = &descriptor.fragment {
-                let fragment_module = match self.shader_cache.get(
-                    &self.device,
-                    id,
-                    &fragment.shader,
-                    &fragment.shader_defs,
-                ) {
-                    Ok(module) => module,
-                    Err(err) => {
-                        state.state = CachedPipelineState::Err(err);
-                        self.waiting_pipelines.insert(id);
-                        continue;
-                    }
-                };
-                Some((
-                    fragment_module,
-                    fragment.entry_point.deref(),
-                    &fragment.targets,
-                ))
-            } else {
-                None
-            };
-
-            let vertex_buffer_layouts = descriptor
-                .vertex
-                .buffers
-                .iter()
-                .map(|layout| VertexBufferLayout {
-                    array_stride: layout.array_stride,
-                    attributes: &layout.attributes,
-                    step_mode: layout.step_mode,
-                })
-                .collect::<Vec<_>>();
-
-            let layout = if let Some(layout) = &descriptor.layout {
-                Some(self.layout_cache.get(&self.device, layout))
-            } else {
-                None
-            };
-
-            let descriptor = RawRenderPipelineDescriptor {
-                multiview: None,
-                depth_stencil: descriptor.depth_stencil.clone(),
-                label: descriptor.label.as_deref(),
-                layout,
-                multisample: descriptor.multisample,
-                primitive: descriptor.primitive,
-                vertex: RawVertexState {
-                    buffers: &vertex_buffer_layouts,
-                    entry_point: descriptor.vertex.entry_point.deref(),
-                    module: &vertex_module,
-                },
-                fragment: fragment_data
-                    .as_ref()
-                    .map(|(module, entry_point, targets)| RawFragmentState {
-                        entry_point,
-                        module,
-                        targets,
-                    }),
-            };
-
-            let pipeline = self.device.create_render_pipeline(&descriptor);
-            state.state = CachedPipelineState::Ok(pipeline);
+            if let CachedPipelineState::Err(_) = pipeline.state {
+                self.waiting_pipelines.insert(id);
+            }
         }
+
+        self.pipelines = pipelines;
     }
 
     pub(crate) fn process_pipeline_queue_system(mut cache: ResMut<Self>) {
@@ -393,7 +528,7 @@ impl RenderPipelineCache {
         shaders: Res<Assets<Shader>>,
         mut events: EventReader<AssetEvent<Shader>>,
     ) {
-        let mut cache = world.get_resource_mut::<Self>().unwrap();
+        let mut cache = world.resource_mut::<Self>();
         for event in events.iter() {
             match event {
                 AssetEvent::Created { handle } | AssetEvent::Modified { handle } => {
@@ -464,15 +599,22 @@ fn log_shader_error(source: &ProcessedShader, error: &AsModuleDescriptorError) {
                 let config = term::Config::default();
                 let mut writer = term::termcolor::Ansi::new(Vec::new());
 
-                let diagnostic = Diagnostic::error().with_labels(
-                    error
-                        .spans()
-                        .map(|(span, desc)| {
-                            Label::primary((), span.to_range().unwrap())
-                                .with_message(desc.to_owned())
-                        })
-                        .collect(),
-                );
+                let diagnostic = Diagnostic::error()
+                    .with_message(error.to_string())
+                    .with_labels(
+                        error
+                            .spans()
+                            .map(|(span, desc)| {
+                                Label::primary((), span.to_range().unwrap())
+                                    .with_message(desc.to_owned())
+                            })
+                            .collect(),
+                    )
+                    .with_notes(
+                        ErrorSources::of(error)
+                            .map(|source| source.to_string())
+                            .collect(),
+                    );
 
                 term::emit(&mut writer, &config, &files, &diagnostic).expect("cannot write error");
 
@@ -490,3 +632,41 @@ fn log_shader_error(source: &ProcessedShader, error: &AsModuleDescriptorError) {
         }
     }
 }
+
+#[derive(Error, Debug)]
+pub enum PipelineCacheError {
+    #[error(
+        "Pipeline cound not be compiled because the following shader is not loaded yet: {0:?}"
+    )]
+    ShaderNotLoaded(Handle<Shader>),
+    #[error(transparent)]
+    ProcessShaderError(#[from] ProcessShaderError),
+    #[error("{0}")]
+    AsModuleDescriptorError(AsModuleDescriptorError, ProcessedShader),
+    #[error("Shader import not yet available.")]
+    ShaderImportNotYetAvailable,
+    #[error("Could not create shader module: {0}")]
+    CreateShaderModule(String),
+}
+
+struct ErrorSources<'a> {
+    current: Option<&'a (dyn std::error::Error + 'static)>,
+}
+
+impl<'a> ErrorSources<'a> {
+    fn of(error: &'a dyn std::error::Error) -> Self {
+        Self {
+            current: error.source(),
+        }
+    }
+}
+
+impl<'a> Iterator for ErrorSources<'a> {
+    type Item = &'a (dyn std::error::Error + 'static);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        let current = self.current;
+        self.current = self.current.and_then(std::error::Error::source);
+        current
+    }
+}
